/*-----------------------------------------------------------------------------
 * Name:    Comparator_LPC812-LPCXpresso.c
 * Purpose: Comparator interface for LPC812-LPCXpresso evaluation board
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2018 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "RTE_Components.h"             // Component selection
#include  CMSIS_device_header           // Device header
#include "Comparator_LPC812-LPCXpresso.h"

#define CMP_LAD_MAX       (0x1F)
#define CMP_LAD_MID      ((CMP_LAD_MAX + 1) / 2)

/**
  \fn          int32_t Comparator_Initialize (void)
  \brief       Initialize Analog Comparator
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t Comparator_Initialize (void) {

  /* configure PIN P0.0 for ACMP_I1 */
  SYSCON->SYSAHBCLKCTRL |= (( 1UL <<  6) |        /* enable clock for GPIO */
                            ( 1UL <<  7) );       /* enable clock for SWM */

  IOCON->PIO[0]   &= ~( 3UL <<  3);               /* no pull up/down */
  GPIO->DIR[0] &= ~( 1UL <<  0);                  /* configure GPIO as input */

  SWM0->PINENABLE0 &= ~( 1UL <<  0);              /* P0.0 is ACMP_I1 */

  /* configure CMP */
  SYSCON->PDRUNCFG      &= ~( 1UL <<  3);         /* Enable power to BOD block */
  SYSCON->PDRUNCFG      &= ~( 1UL << 15);         /* Enable power to CMP block */
  SYSCON->SYSAHBCLKCTRL |=  ( 1UL << 19);         /* Enable clock to CMP block */

  SYSCON->PRESETCTRL    &= ~( 1UL << 12);         /* reset CMP */
  SYSCON->PRESETCTRL    |=  ( 1UL << 12);

  ACOMP->CTRL  =  (( 0UL <<  3) |                 /* EDGESEL, Falling edges */
                   ( 0UL <<  6) |                 /* COMPSA, output used directly */
                   ( 1UL <<  8) |                 /* COMP_VP_SEL, ACMP_I1 */
                   ( 0UL << 11) |                 /* COMP_VM_SEL, Voltage ladder output */
                   ( 0UL << 25)  );               /* HYS, none */
  ACOMP->LAD   =  (( 1UL <<  0) |                 /* Voltage ladder enable */
                   (CMP_LAD_MID <<  1) |
                   ( 0UL <<  6)  );               /* LADREF, Supply pin VDD */


  return 0;
}


/**
  \fn          int32_t Comparator_GetLad (void)
  \brief       Get Ladder value
  \returns
   - <b> >=0</b>: Ladder value
   - \b -1: conversion in progress or failed
*/
int16_t Comparator_GetLad (void) {
  uint32_t cnt, i;
  uint32_t ladVal;

  for (cnt = 1; cnt < 5; cnt++) {
    ladVal = (ACOMP->LAD >> 1) & CMP_LAD_MAX;

    if (ACOMP->CTRL & (1UL << 21)) {
      ladVal +=  (CMP_LAD_MID >> cnt);            /* increment ladder value */
    } else {
      ladVal -=  (CMP_LAD_MID >> cnt);            /* decrement ladder value */
    }
    ACOMP->LAD = ((ladVal & CMP_LAD_MAX) << 1) | 1;
    for (i = 0; i < 10; i++)  __NOP();
  }
  ACOMP->LAD = ((CMP_LAD_MID) << 1) | 1;

  return(ladVal);
}
